@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -106,8 +106,6 @@ static const char *my_login_path= 0;
 static char my_defaults_file_buffer[FN_REFLEN];
 static char my_defaults_extra_file_buffer[FN_REFLEN];
 
-static char my_login_file[FN_REFLEN];
-
 static my_bool defaults_already_read= FALSE;
 
 #ifdef WITH_WSREP
@@ -116,11 +114,6 @@ static my_bool defaults_already_read= FALSE;
 char wsrep_defaults_file[FN_REFLEN + 10]={0,};
 char wsrep_defaults_group_suffix[FN_EXTLEN]={0,};
 #endif /* WITH_WREP */
-/* Set to TRUE, if --no-defaults is found. */
-static my_bool found_no_defaults= FALSE;
-
-/* Set to TRUE, when login file is being processed. */
-static my_bool is_login_file= FALSE;
 
 /* Which directories are searched for options (and in which order) */
 
@@ -154,13 +147,16 @@ struct handle_option_ctx
 };
 
 static int search_default_file(Process_option_func func, void *func_ctx,
-			       const char *dir, const char *config_file);
+                               const char *dir, const char *config_file,
+                               my_bool is_login_file);
 static int search_default_file_with_ext(Process_option_func func,
                                         void *func_ctx,
 					const char *dir, const char *ext,
-					const char *config_file, int recursion_level);
-static my_bool mysql_file_getline(char *str, int size, MYSQL_FILE *file);
-
+                                        const char *config_file,
+                                        int recursion_level,
+                                        my_bool is_login_file);
+static my_bool mysql_file_getline(char *str, int size, MYSQL_FILE *file,
+                                  my_bool is_login_file);
 
 /**
   Create the list of default directories.
@@ -236,6 +232,7 @@ fn_expand(const char *filename, char *result_buf)
   func_ctx                    It's context. Usually it is the structure to
                               store additional options.
   default_directories         List of default directories.
+  found_no_defaults           TRUE, if --no-defaults is specified.
 
   DESCRIPTION
     Process the default options from argc & argv
@@ -259,7 +256,8 @@ fn_expand(const char *filename, char *result_buf)
 
 int my_search_option_files(const char *conf_file, int *argc, char ***argv,
                            uint *args_used, Process_option_func func,
-                           void *func_ctx, const char **default_directories)
+                           void *func_ctx, const char **default_directories,
+                           my_bool is_login_file, my_bool found_no_defaults)
 {
   const char **dirs, *forced_default_file, *forced_extra_defaults;
   int error= 0;
@@ -273,10 +271,10 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
                                       (char **) &forced_default_file,
                                       (char **) &forced_extra_defaults,
                                       (char **) &my_defaults_group_suffix,
-                                      (char **) &my_login_path);
+                                      (char **) &my_login_path, found_no_defaults);
 
     if (! my_defaults_group_suffix)
-      my_defaults_group_suffix= getenv(STRINGIFY_ARG(DEFAULT_GROUP_SUFFIX_ENV));
+      my_defaults_group_suffix= getenv("MYSQL_GROUP_SUFFIX");
 
     if (forced_extra_defaults && !defaults_already_read)
     {
@@ -388,14 +386,16 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
   // If conf_file is an absolute path, we only read it
   if (dirname_length(conf_file))
   {
-    if ((error= search_default_file(func, func_ctx, NullS, conf_file)) < 0)
-      goto err;
+    if ((error= search_default_file(func, func_ctx, NullS, conf_file,
+                                    is_login_file)) < 0)
+    goto err;
   }
   // If my defaults file is set (from a previous run), we read it
   else if (my_defaults_file)
   {
     if ((error= search_default_file_with_ext(func, func_ctx, "", "",
-                                             my_defaults_file, 0)) < 0)
+                                             my_defaults_file, 0,
+                                             is_login_file)) < 0)
       goto err;
     if (error > 0)
     {
@@ -410,14 +410,16 @@ int my_search_option_files(const char *conf_file, int *argc, char ***argv,
     {
       if (**dirs)
       {
-	if (search_default_file(func, func_ctx, *dirs, conf_file) < 0)
-	  goto err;
+       if (search_default_file(func, func_ctx, *dirs, conf_file,
+                               is_login_file) < 0)
+        goto err;
       }
       else if (my_defaults_extra_file)
       {
         if ((error= search_default_file_with_ext(func, func_ctx, "", "",
-                                                my_defaults_extra_file, 0)) < 0)
-	  goto err;				/* Fatal error */
+                                                 my_defaults_extra_file, 0,
+                                                 is_login_file)) < 0)
+          goto err;				/* Fatal error */
         if (error > 0)
         {
           fprintf(stderr, "Could not open required defaults file: %s\n",
@@ -505,7 +507,8 @@ int get_defaults_options(int argc, char **argv,
                          char **defaults,
                          char **extra_defaults,
                          char **group_suffix,
-                         char **login_path)
+                         char **login_path,
+                         my_bool found_no_defaults)
 {
   int org_argc= argc, prev_argc= 0, default_option_count= 0;
   *defaults= *extra_defaults= *group_suffix= *login_path= 0;
@@ -589,6 +592,8 @@ int load_defaults(const char *conf_file, const char **groups,
   return my_load_defaults(conf_file, groups, argc, argv, &default_directories);
 }
 
+/** A global to turn off or on reading the mylogin file. On by default */
+my_bool my_defaults_read_login_file= TRUE;
 /*
   Read options from configurations files
 
@@ -639,6 +644,8 @@ int my_load_defaults(const char *conf_file, const char **groups,
   char *ptr,**res;
   struct handle_option_ctx ctx;
   const char **dirs;
+  char my_login_file[FN_REFLEN];
+  my_bool found_no_defaults= false;
   uint args_sep= my_getopt_use_args_separator ? 1 : 0;
   DBUG_ENTER("load_defaults");
 
@@ -668,23 +675,26 @@ int my_load_defaults(const char *conf_file, const char **groups,
 
   if ((error= my_search_option_files(conf_file, argc, argv,
                                      &args_used, handle_default_option,
-                                     (void *) &ctx, dirs)))
+                                     (void *) &ctx, dirs, false, found_no_defaults)))
   {
+    delete_dynamic(&args);
     free_root(&alloc,MYF(0));
     DBUG_RETURN(error);
   }
 
-  /* Read options from login group. */
-  is_login_file= TRUE;
-  if (my_default_get_login_file(my_login_file, sizeof(my_login_file)) &&
-      (error= my_search_option_files(my_login_file,argc, argv, &args_used,
-                                     handle_default_option, (void *) &ctx,
-                                     dirs)))
+  if (my_defaults_read_login_file)
   {
-    free_root(&alloc,MYF(0));
-    DBUG_RETURN(error);
+    /* Read options from login group. */
+    if (my_default_get_login_file(my_login_file, sizeof(my_login_file)) &&
+      (error= my_search_option_files(my_login_file, argc, argv, &args_used,
+                                     handle_default_option, (void *) &ctx,
+                                     dirs, true, found_no_defaults)))
+    {
+      delete_dynamic(&args);
+      free_root(&alloc, MYF(0));
+      DBUG_RETURN(error);
+    }
   }
-  is_login_file= FALSE;
 
   /*
     Here error contains <> 0 only if we have a fully specified conf_file
@@ -692,7 +702,10 @@ int my_load_defaults(const char *conf_file, const char **groups,
   */
   if (!(ptr=(char*) alloc_root(&alloc,sizeof(alloc)+
 			       (args.elements + *argc + 1 + args_sep) *sizeof(char*))))
+  {
+    delete_dynamic(&args);
     goto err;
+  }
   res= (char**) (ptr+sizeof(alloc));
 
   /* copy name + found arguments + command line arguments to new array */
@@ -752,7 +765,7 @@ int my_load_defaults(const char *conf_file, const char **groups,
  err:
   fprintf(stderr,"Fatal error in defaults handling. Program aborted\n");
   exit(1);
-  return 0;					/* Keep compiler happy */
+  DBUG_RETURN(0); /* Keep compiler happy */
 }
 
 
@@ -767,7 +780,8 @@ void free_defaults(char **argv)
 static int search_default_file(Process_option_func opt_handler,
                                void *handler_ctx,
 			       const char *dir,
-			       const char *config_file)
+                               const char *config_file,
+                               my_bool is_login_file)
 {
   char **ext;
   const char *empty_list[]= { "", 0 };
@@ -779,7 +793,7 @@ static int search_default_file(Process_option_func opt_handler,
     int error;
     if ((error= search_default_file_with_ext(opt_handler, handler_ctx,
                                              dir, *ext,
-					     config_file, 0)) < 0)
+                                             config_file, 0, is_login_file)) < 0)
       return error;
   }
   return 0;
@@ -851,6 +865,7 @@ static char *get_argument(const char *keyword, size_t kwlen,
     group			groups to read
     recursion_level             the level of recursion, got while processing
                                 "!include" or "!includedir"
+    is_login_file               TRUE, when login file is being processed.
 
   RETURN
     0   Success
@@ -863,7 +878,8 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
                                         const char *dir,
                                         const char *ext,
                                         const char *config_file,
-                                        int recursion_level)
+                                        int recursion_level,
+                                        my_bool is_login_file)
 {
   char name[FN_REFLEN + 10], buff[4096], curr_gr[4096], *ptr, *end, **tmp_ext;
   char *value, option[4096+2], tmp[FN_REFLEN];
@@ -892,7 +908,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
   }
   fn_format(name,name,"","",4);
 
-  if ((rc= check_file_permissions(name)) < 2)
+  if ((rc= check_file_permissions(name, is_login_file)) < 2)
     return (int) rc;
 
   if (is_login_file)
@@ -912,7 +928,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
     strncpy(wsrep_defaults_file, name, sizeof(wsrep_defaults_file) - 1);
 #endif /* WITH_WSREP */
 
-  while (mysql_file_getline(buff, sizeof(buff) - 1, fp))
+  while (mysql_file_getline(buff, sizeof(buff) - 1, fp, is_login_file))
   {
     line++;
     /* Ignore comment and empty lines */
@@ -973,7 +989,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
                       MY_UNPACK_FILENAME | MY_SAFE_PATH);
 
             search_default_file_with_ext(opt_handler, handler_ctx, "", "", tmp,
-                                         recursion_level + 1);
+                                         recursion_level + 1, is_login_file);
           }
         }
 
@@ -988,7 +1004,7 @@ static int search_default_file_with_ext(Process_option_func opt_handler,
 	  goto err;
 
         search_default_file_with_ext(opt_handler, handler_ctx, "", "", ptr,
-                                     recursion_level + 1);
+                                     recursion_level + 1, is_login_file);
       }
 
       continue;
@@ -1150,15 +1166,17 @@ static char *remove_end_comment(char *ptr)
   of scrambled login file, the line read is first
   decrypted and then returned.
 
-  @param str  [out]       Buffer to store the read text.
-  @param size [in]        At max, size-1 bytes to be read.
-  @param file [in]        Source file.
+  @param str           [out]  Buffer to store the read text.
+  @param size          [in]   At max, size-1 bytes to be read.
+  @param file          [in]   Source file.
+  @param is_login_file [in]   TRUE, when login file is being processed.
 
   @return 1               Success
           0               Error
 */
 
-static my_bool mysql_file_getline(char *str, int size, MYSQL_FILE *file)
+static my_bool mysql_file_getline(char *str, int size, MYSQL_FILE *file,
+                                  my_bool is_login_file)
 {
   uchar cipher[4096], len_buf[MAX_CIPHER_STORE_LEN];
   static unsigned char my_key[LOGIN_KEY_LEN];
@@ -1297,7 +1315,7 @@ static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs)
   char buf[FN_REFLEN];
   size_t len;
   char *p;
-  my_bool err __attribute__((unused));
+  my_bool err MY_ATTRIBUTE((unused));
 
   len= normalize_dirname(buf, dir);
   if (!(p= strmake_root(alloc, buf, len)))
@@ -1471,13 +1489,14 @@ int my_default_get_login_file(char *file_name, size_t file_name_size)
 /**
   Check file permissions of the option file.
 
-  @param file_name [in]       Name of the option file.
+  @param file_name     [in]   Name of the option file.
+  @param is_login_file [in]   TRUE, when login file is being processed.
 
   @return  0 - Non-allowable file permissions.
            1 - Failed to stat.
            2 - Success.
 */
-int check_file_permissions(const char *file_name)
+int check_file_permissions(const char *file_name, my_bool is_login_file)
 {
 #if !defined(__WIN__)
   MY_STAT stat_info;
