@@ -32,6 +32,8 @@
 #include "sp_head.h"
 #include "sql_show.h"                // append_definer, append_identifier
 
+#include "wsrep_thd.h"
+
 /**
   @addtogroup Event_Scheduler
   @{
@@ -1444,6 +1446,25 @@ Event_job_data::execute(THD *thd, bool drop)
       There is no pre-locking and therefore there should be no
       tables open and locked left after execute_procedure.
     */
+
+#ifdef WITH_WSREP
+    /*
+      If the thread has been killed, rollback the operation
+      properly (for WSREP).
+    */
+    if (WSREP(thd))
+    {
+      mysql_mutex_lock(&thd->LOCK_wsrep_thd);
+      if (thd->wsrep_conflict_state == MUST_ABORT)
+      {
+        wsrep_client_rollback(thd);
+
+        WSREP_DEBUG("abort the event in exec query state, avoiding autocommit");
+      }
+      mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
+    }
+#endif /* WITH_WSREP */
+
   }
 
 end:
@@ -1482,24 +1503,36 @@ end:
       thd->tx_read_only= false;
 
 #ifdef WITH_WSREP
-      if (WSREP(thd)) {
-          // sql_print_information("sizeof(LEX) = %d", sizeof(struct LEX));
-          // sizeof(LEX) = 4512, so it's relatively safe to allocate it on stack.
-          LEX lex;
-          LEX* saved = thd->lex;
-          lex.sql_command = SQLCOM_DROP_EVENT;
-          thd->lex = &lex;
-          WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL);
-          thd->lex = saved;
+      /* 
+         This code is processing event execution and does not have client 
+         connection. Here, event execution will now execute a prepared 
+         DROP EVENT statement, but thd->lex->sql_command is set to 
+         SQLCOM_CREATE_PROCEDURE 
+         DROP EVENT will be logged in binlog, and we have to
+         replicate it to make all nodes have consistent event definitions
+         Wsrep DDL replication is triggered inside Events::drop_event(), 
+         and here we need to prepare the THD so that DDL replication is
+         possible, essentially it requires setting sql_command to 
+         SQLCOMM_DROP_EVENT, we will switch sql_command for the duration
+         of DDL replication only.
+      */
+      enum_sql_command sql_command_save= SQLCOM_END;
+      if (WSREP(thd))
+      {
+        sql_command_save= thd->lex->sql_command;
+        thd->lex->sql_command = SQLCOM_DROP_EVENT;
       }
 #endif
-     
+
       ret= Events::drop_event(thd, dbname, name, FALSE);
 
 #ifdef WITH_WSREP
-      WSREP_TO_ISOLATION_END;
-  error:
-#endif      
+      if (WSREP(thd))
+      {
+        WSREP_TO_ISOLATION_END;
+        thd->lex->sql_command = sql_command_save;
+      }
+#endif
       thd->tx_read_only= save_tx_read_only;
       thd->security_ctx->master_access= saved_master_access;
     }
