@@ -201,7 +201,11 @@ lock_tables_check(THD *thd, TABLE **tables, uint count, uint flags)
     if (!(flags & MYSQL_LOCK_IGNORE_GLOBAL_READ_ONLY) && !t->s->tmp_table)
     {
       if (t->reginfo.lock_type >= TL_WRITE_ALLOW_WRITE &&
-          enforce_ro && opt_readonly && !thd->slave_thread)
+          enforce_ro && opt_readonly && !thd->slave_thread
+#ifdef WITH_WSREP
+          && !thd->wsrep_applier
+#endif /* WITH_WSREP */
+         )
       {
         my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0),
                  opt_super_readonly ? "--read-only (super)" : "--read-only");
@@ -354,7 +358,7 @@ end:
 
 static int lock_external(THD *thd, TABLE **tables, uint count)
 {
-  reg1 uint i;
+  uint i;
   int lock_type,error;
   DBUG_ENTER("lock_external");
 
@@ -489,7 +493,7 @@ void mysql_lock_remove(THD *thd, MYSQL_LOCK *locked,TABLE *table)
 {
   if (locked)
   {
-    reg1 uint i;
+    uint i;
     for (i=0; i < locked->table_count; i++)
     {
       if (locked->table[i] == table)
@@ -1144,7 +1148,11 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
     int rcode;
     WSREP_DEBUG("running implicit desync for node");
     rcode = wsrep->desync(wsrep);
-    if (rcode != WSREP_OK)
+    if (rcode == WSREP_TRX_FAIL && strcmp(wsrep_cluster_status, "Primary") != 0)
+    {
+      WSREP_DEBUG("desync failed while non-Primary, ignoring failure");
+    }
+    else if (rcode != WSREP_OK)
     {
       WSREP_WARN("FTWRL desync failed %d for schema: %s, query: %s",
                  rcode, (thd->db ? thd->db : "(null)"), WSREP_QUERY(thd));
@@ -1176,23 +1184,11 @@ bool Global_read_lock::make_global_read_lock_block_commit(THD *thd)
 */
 bool Global_read_lock::wsrep_pause()
 {
-  mysql_mutex_lock(&LOCK_wsrep_pause_count);
-
-  /* Provider already paused. Avoid re-pausing it.
-  Just increment the count. */
-  if (wsrep_pause_count > 0)
-  {
-     ++wsrep_pause_count;
-     mysql_mutex_unlock(&LOCK_wsrep_pause_count);
-     return(TRUE);
-  }
-
   wsrep_seqno_t ret = wsrep->pause(wsrep);
   if (ret >= 0)
   {
     wsrep_locked_seqno= ret;
-    assert(wsrep_pause_count == 0);
-    ++wsrep_pause_count;
+    pause_provider(true);
   }
   else if (ret != -ENOSYS) /* -ENOSYS - no provider */
   {
@@ -1201,11 +1197,9 @@ bool Global_read_lock::wsrep_pause()
     /* m_mdl_blocks_commits_lock is always NULL here */
     wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
     my_error(ER_LOCK_DEADLOCK, MYF(0));
-    mysql_mutex_unlock(&LOCK_wsrep_pause_count);
     return FALSE;
   }
 
-  mysql_mutex_unlock(&LOCK_wsrep_pause_count);
   return TRUE;
 }
 
@@ -1216,46 +1210,41 @@ bool Global_read_lock::wsrep_pause()
   @retval False  Failed to pause the provider, wsrep_locked_seqno is reset.
   @retval True   Provider has been paused.
 */
-wsrep_status_t Global_read_lock::wsrep_resume(bool ignore_if_resumed)
+wsrep_status_t Global_read_lock::wsrep_resume()
 {
-  wsrep_status_t ret(WSREP_OK);
+  wsrep_status_t ret;
 
-  mysql_mutex_lock(&LOCK_wsrep_pause_count);
+  wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
+  ret = wsrep->resume(wsrep);
 
-  if (ignore_if_resumed && wsrep_pause_count == 0)
+  if (ret != WSREP_OK)
   {
-    mysql_mutex_unlock(&LOCK_wsrep_pause_count);
-    return ret;
+    WSREP_WARN("Failed to resume provider: %d", ret);
+  }
+  else
+  {
+    pause_provider(false);
   }
 
-  assert(wsrep_pause_count > 0);
-  if (--wsrep_pause_count == 0)
-  {
-    wsrep_locked_seqno= WSREP_SEQNO_UNDEFINED;
-    ret = wsrep->resume(wsrep);
-
-    if (ret)
-      WSREP_WARN("Failed to resume provider: %d", ret);
-  }
-
-  mysql_mutex_unlock(&LOCK_wsrep_pause_count);
   return ret;
 }
 
-bool Global_read_lock::wsrep_pause_once()
+bool Global_read_lock::wsrep_pause_once(bool *already_paused)
 {
-    provider_paused= true;
-    return wsrep_pause();
+    if (!provider_paused)
+    {
+      *already_paused= FALSE;
+      return wsrep_pause();
+    }
+    *already_paused= TRUE;
+    return TRUE;
 }
 
 wsrep_status_t Global_read_lock::wsrep_resume_once(void)
 {
-  if (provider_paused)
-  {
-    provider_paused= false;
-    return wsrep_resume(true);
-  }
-  return WSREP_OK;
+    if (provider_paused)
+      return wsrep_resume();
+    return WSREP_OK;
 }
 
 #endif /* WITH_WSREP */
@@ -1334,35 +1323,19 @@ void Global_backup_lock::release(THD *thd)
 
 void Global_backup_lock::set_explicit_locks_duration(THD *thd)
 {
-  bool should_own;
-
   DBUG_ENTER("Global_backup_lock::set_explicit_lock_duration");
 
   if (m_lock)
-  {
-    should_own= true;
     thd->mdl_context.set_lock_duration(m_lock, MDL_EXPLICIT);
-  }
-  else
-  {
-    should_own= false;
-  }
 
-  DBUG_ASSERT(should_own ==
+  DBUG_ASSERT((m_lock != NULL) ==
               thd->mdl_context.is_lock_owner(m_namespace, "", "",
                                              MDL_SHARED));
 
   if (m_prot_lock)
-  {
-    should_own= true;
     thd->mdl_context.set_lock_duration(m_prot_lock, MDL_EXPLICIT);
-  }
-  else
-  {
-    should_own= false;
-  }
 
-  DBUG_ASSERT(should_own ==
+  DBUG_ASSERT((m_prot_lock != NULL) ==
               thd->mdl_context.is_lock_owner(m_namespace, "", "",
                                              MDL_INTENTION_EXCLUSIVE));
 
