@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -902,6 +902,11 @@ public:
   {
     return &link_next;
   }
+
+  /// @return true if query block references full-text functions
+  bool has_ft_funcs() const
+  { return ftfunc_list->elements > 0; }
+
   void invalidate();
   void mark_as_dependent(st_select_lex *last);
 
@@ -1090,6 +1095,8 @@ struct st_sp_chistics
 
 extern const LEX_STRING null_lex_str;
 extern const LEX_STRING empty_lex_str;
+extern const LEX_CSTRING null_lex_cstr;
+extern const LEX_CSTRING empty_lex_cstr;
 
 struct st_trg_chistics
 {
@@ -1421,6 +1428,11 @@ public:
     return get_stmt_unsafe_flags() != 0;
   }
 
+  inline bool is_stmt_unsafe(enum_binlog_stmt_unsafe unsafe)
+  {
+    return binlog_stmt_flags & (1 << unsafe);
+  }
+
   /**
     Flag the current (top-level) statement as unsafe.
     The flag will be reset after the statement has finished.
@@ -2444,6 +2456,17 @@ struct LEX: public Query_tables_list
   bool is_set_password_sql;
   bool contains_plaintext_password;
 
+  /*
+    Compression dictionary name (in column definition)
+    CREATE TABLE t1(
+      ...
+      <column_name> BLOB COLUMN_FORMAT COMPRESSED
+        WITH COMPRESSION_DICTIONARY <dict>
+      ...
+    );
+  */
+  LEX_CSTRING zip_dict_name;
+
 private:
   bool m_broken; ///< see mark_broken()
   /// Current SP parsing context.
@@ -2547,6 +2570,8 @@ public:
   
   bool escape_used;
   bool is_lex_started; /* If lex_start() did run. For debugging. */
+  /// Set to true while resolving values in ON DUPLICATE KEY UPDATE clause
+  bool in_update_value_clause;
 
   /*
     The set of those tables whose fields are referenced in all subqueries
