@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -135,10 +135,12 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
     }
 
   if ((options & REFRESH_SLOW_LOG) && opt_slow_log)
-    logger.flush_slow_log();
+    if (logger.flush_slow_log())
+      result= 1;
 
   if ((options & REFRESH_GENERAL_LOG) && opt_log)
-    logger.flush_general_log();
+    if (logger.flush_general_log())
+      result= 1;
 
   if (options & REFRESH_ENGINE_LOG)
     if (ha_flush_logs(NULL))
@@ -178,7 +180,7 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
       if (active_mi != NULL)
       {
         mysql_mutex_lock(&active_mi->data_lock);
-        if (rotate_relay_log(active_mi))
+        if (rotate_relay_log(active_mi, true/*need_log_space_lock=true*/))
           *write_to_binlog= -1;
         mysql_mutex_unlock(&active_mi->data_lock);
       }
@@ -345,11 +347,11 @@ bool reload_acl_and_cache(THD *thd, unsigned long options,
         }
       }
 
-      if (thd && thd->wsrep_applier)
+      if (thd && (thd->wsrep_applier || thd->slave_thread))
       {
         /*
-          In case of applier thread, do not wait for table share(s) to be
-          removed from table definition cache.
+          In case of wsrep-applier/mysql-slave thread, do not wait for table
+          share(s) to be removed from table definition cache.
 
           This is important otherwise it can lead to a deadlock in following
           scenario:
