@@ -438,14 +438,30 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
   if (!trust_function_creators                                && 
       (WSREP_EMULATE_BINLOG(thd) || mysql_bin_log.is_open())  &&
       !(thd->security_ctx->master_access & SUPER_ACL))
+  {
+    /*
+      If WSREP is enabled, then we are ALWAYS doing binlog
+      replication of some sort, and we always require the SUPER
+      privilege (or trust_function_creators).
+      So there is no need to mention anything about the binlog.
+    */
+    if (WSREP(thd))
+      my_message(ER_BINLOG_CREATE_ROUTINE_NEED_SUPER,
+                "You do not have the SUPER privilege"
+                " (you *might* want to use the less safe log_bin_trust_function_creators variable)",
+                MYF(0));
+    else
+      my_error(ER_BINLOG_CREATE_ROUTINE_NEED_SUPER, MYF(0));
+    DBUG_RETURN(TRUE);
+  }
 #else
   if (!trust_function_creators && mysql_bin_log.is_open() &&
       !(thd->security_ctx->master_access & SUPER_ACL))
-#endif /* WITH_WSREP */
   {
     my_error(ER_BINLOG_CREATE_ROUTINE_NEED_SUPER, MYF(0));
     DBUG_RETURN(TRUE);
   }
+#endif /* WITH_WSREP */
 
   if (!create)
   {
@@ -467,7 +483,11 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     */
     thd->lex->sql_command= backup.sql_command;
 
+#ifdef WITH_WSREP
+    if (opt_readonly && enforce_ro && !thd->slave_thread && !thd->wsrep_applier)
+#else
     if (opt_readonly && enforce_ro && !thd->slave_thread)
+#endif /* WITH_WSREP */
     {
       my_error(ER_OPTION_PREVENTS_STATEMENT, MYF(0),
                opt_super_readonly ? "--read-only (super)" : "--read-only");
@@ -491,6 +511,11 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
       result= FALSE;
       /* Still, we need to log the query ... */
       stmt_query.append(thd->query(), thd->query_length());
+#ifdef WITH_WSREP
+      /* Table doesn't exist but query is still being logged
+      so replicate a query with NULL construct. */
+      WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, tables)
+#endif /* WITH_WSREP */
       goto end;
     }
   }
@@ -521,6 +546,8 @@ bool mysql_create_or_drop_trigger(THD *thd, TABLE_LIST *tables, bool create)
     goto end;
   }
 
+  WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, tables)
+
   /* We also don't allow creation of triggers on views. */
   tables->required_type= FRMTYPE_TABLE;
   /*
@@ -615,6 +642,10 @@ end:
     my_ok(thd);
 
   DBUG_RETURN(result);
+#ifdef WITH_WSREP
+ error:
+  DBUG_RETURN(TRUE);
+#endif /* WITH_WSREP */
 }
 
 
@@ -1818,7 +1849,7 @@ bool Table_triggers_list::drop_all_triggers(THD *thd, char *db, char *name)
   bool result= 0;
   DBUG_ENTER("drop_all_triggers");
 
-  memset(&table, 0, sizeof(table));
+  memset(static_cast<void*>(&table), 0, sizeof(table));
   init_sql_alloc(&table.mem_root, 8192, 0);
 
   if (Table_triggers_list::check_n_load(thd, db, name, &table, 1))
@@ -2038,7 +2069,7 @@ bool Table_triggers_list::change_table_name(THD *thd, const char *db,
   LEX_STRING *err_trigname;
   DBUG_ENTER("change_table_name");
 
-  memset(&table, 0, sizeof(table));
+  memset(static_cast<void*>(&table), 0, sizeof(table));
   init_sql_alloc(&table.mem_root, 8192, 0);
 
   /*
